
#include <gtkmm/scale.h>
#include <gtkmm/private/scale_p.h>

// -*- c++ -*-
/* $Id: scale.ccg,v 1.10 2001/12/04 15:56:30 murrayc Exp $ */

/* 
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtkmm/adjustment.h>

#include <gtk/gtkscale.h>
#include <gtk/gtkhscale.h>
#include <gtk/gtkvscale.h>
#include <math.h>

namespace Gtk
{

int Scale::calc_digits_(double step)
{
 int digits = 0;

 //Copied from gtk_[h|v]scale_new_with_range():
 if (fabs (step) >= 1.0 || step == 0.0)
    digits = 0;
  else {
    digits = abs ((int) floor (log10 (fabs (step))));
    if (digits > 5)
      digits = 5;
  }

  return digits;
}

VScale::VScale(double min, double max, double step)
: Scale((GtkScale*)(g_object_new(get_type(),0)))
{
  Adjustment* adjustment = manage(new Adjustment(min, min, max, step, 10 * step, step));
  // The adjustment will be destroyed along with the object
  set_adjustment(*adjustment);

  set_digits( calc_digits_(step) );
}

VScale::VScale(Adjustment& adjustment)
: Scale((GtkScale*)(g_object_new(get_type(),0)))
{
  set_adjustment(adjustment);
}

VScale::VScale()
: Scale((GtkScale*)(g_object_new(get_type(),0)))
{
  Adjustment* adjustment = manage(new Adjustment(0.0, 0.0, 0.0,
						  0.0, 0.0, 0.0));
  // The adjustment will be destroyed along with the object
  set_adjustment(*adjustment);
}


HScale::HScale(double min, double max, double step)
: Scale((GtkScale*)(g_object_new(get_type(),0)))
{
  Adjustment* adjustment = manage(new Adjustment(min, min, max, step, 10 * step, step));
  // The adjustment will be destroyed along with the object
  set_adjustment(*adjustment);

  set_digits( calc_digits_(step) );
}

HScale::HScale()
: Scale((GtkScale*)(g_object_new(get_type(),0)))
{
  Adjustment* adjustment = manage(new Adjustment(0.0, 0.0, 0.0,
						  0.0, 0.0, 0.0));
  // The adjustment will be destroyed along with the object
  set_adjustment(*adjustment);
}

HScale::HScale(Adjustment& adjustment)
  : Scale((GtkScale*)(g_object_new(get_type(),0)))
{
  set_adjustment(adjustment);
}


} /* namespace Gtk */


namespace
{


gchar* Scale_signal_format_value_callback(GtkScale*, gdouble p0,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot1<Glib::ustring,double> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    return g_strdup(((*(SlotType::Proxy)(slot->proxy_))
        (p0
, slot)).c_str());
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }

  typedef gchar* RType;
  return RType();
}

gchar* Scale_signal_format_value_notify_callback(GtkScale*, gdouble p0,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot1<void,double> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (p0
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }

  typedef gchar* RType;
  return RType();
}


const Glib::SignalProxyInfo Scale_signal_format_value_info =
{
  "format_value",
  (GCallback) &Scale_signal_format_value_callback,
  (GCallback) &Scale_signal_format_value_notify_callback
};

} // anonymous namespace


namespace Glib
{

Gtk::Scale* wrap(GtkScale* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gtk::Scale *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gtk
{


/* The *_Class implementation: */

GType Scale_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_scale_get_type(), "gtkmm__Scale", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void Scale_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->draw_value = &draw_value_vfunc_callback;
  klass->format_value = &format_value_callback;
}


void Scale_Class::draw_value_vfunc_callback(GtkScale* self)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->draw_value();
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->draw_value)
      (*base->draw_value)(self);
  }
}


gchar* Scale_Class::format_value_callback(GtkScale* self, gdouble p0)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      return g_strdup(obj->on_format_value(p0
).c_str());
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->format_value)
      (*base->format_value)(self, p0);
  }

  typedef gchar* RType;
  return RType();
}


Glib::ObjectBase* Scale_Class::wrap_new(GObject* o)
{
  return manage(new Scale((GtkScale*)(o)));
}


/* The implementation: */

Scale::Scale(GtkScale* castitem)
: Gtk::Range((GtkRange*)(castitem))
{}

Scale::~Scale()
{
  destroy_();
}

Scale::CppClassType Scale::scale_class_; //Initialize static member.

GType Scale::get_type()
{
  return scale_class_.get_type();
}

GType Scale::get_base_type()
{
  return gtk_scale_get_type();
}

Scale::Scale()
: Gtk::Range((GtkRange*)(g_object_new(get_type(), (char*)0)))
{}

void Scale::set_digits(int digits)
{
  gtk_scale_set_digits(gobj(), digits);
}

int Scale::get_digits() const
{
  return gtk_scale_get_digits(const_cast<GtkScale*>(gobj()));
}

void Scale::set_draw_value(int draw_value)
{
  gtk_scale_set_draw_value(gobj(), draw_value);
}

int Scale::get_draw_value() const
{
  return gtk_scale_get_draw_value(const_cast<GtkScale*>(gobj()));
}

void Scale::set_value_pos(PositionType pos)
{
  gtk_scale_set_value_pos(gobj(), ((GtkPositionType)(pos)));
}

PositionType Scale::get_value_pos() const
{
  return ((PositionType)(gtk_scale_get_value_pos(const_cast<GtkScale*>(gobj()))));
}


Glib::SignalProxy1<Glib::ustring,double> Scale::signal_format_value()
{
  return Glib::SignalProxy1<Glib::ustring,double>(this, &Scale_signal_format_value_info);
}


Glib::PropertyProxy<int> Scale::property_digits()
{
  return Glib::PropertyProxy<int>(this, "digits");
}

Glib::PropertyProxy<bool> Scale::property_draw_value()
{
  return Glib::PropertyProxy<bool>(this, "draw_value");
}

Glib::PropertyProxy<PositionType> Scale::property_value_pos()
{
  return Glib::PropertyProxy<PositionType>(this, "value_pos");
}


Glib::ustring Gtk::Scale::on_format_value(double value)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->format_value)
    return Glib::convert_const_gchar_ptr_to_ustring((*base->format_value)(gobj(),value));

  typedef Glib::ustring RType;
  return RType();
}


void Gtk::Scale::draw_value()
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->draw_value)
    (*base->draw_value)(gobj());
}


} // namespace Gtk


namespace Glib
{

Gtk::VScale* wrap(GtkVScale* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gtk::VScale *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gtk
{


/* The *_Class implementation: */

GType VScale_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_vscale_get_type(), "gtkmm__VScale", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void VScale_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
}


Glib::ObjectBase* VScale_Class::wrap_new(GObject* o)
{
  return manage(new VScale((GtkVScale*)(o)));
}


/* The implementation: */

VScale::VScale(GtkVScale* castitem)
: Gtk::Scale((GtkScale*)(castitem))
{}

VScale::~VScale()
{
  destroy_();
}

VScale::CppClassType VScale::vscale_class_; //Initialize static member.

GType VScale::get_type()
{
  return vscale_class_.get_type();
}

GType VScale::get_base_type()
{
  return gtk_vscale_get_type();
}


} // namespace Gtk


namespace Glib
{

Gtk::HScale* wrap(GtkHScale* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gtk::HScale *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gtk
{


/* The *_Class implementation: */

GType HScale_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_hscale_get_type(), "gtkmm__HScale", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void HScale_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
}


Glib::ObjectBase* HScale_Class::wrap_new(GObject* o)
{
  return manage(new HScale((GtkHScale*)(o)));
}


/* The implementation: */

HScale::HScale(GtkHScale* castitem)
: Gtk::Scale((GtkScale*)(castitem))
{}

HScale::~HScale()
{
  destroy_();
}

HScale::CppClassType HScale::hscale_class_; //Initialize static member.

GType HScale::get_type()
{
  return hscale_class_.get_type();
}

GType HScale::get_base_type()
{
  return gtk_hscale_get_type();
}


} // namespace Gtk

