
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/private/scrolledwindow_p.h>

// -*- c++ -*-
/* $Id: scrolledwindow.ccg,v 1.10 2002/03/21 12:44:01 murrayc Exp $ */

/* 
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtkmm/scrollbar.h>
#include <gtkmm/viewport.h>
#include <gtkmm/adjustment.h>
#include <gtk/gtkscrolledwindow.h>


namespace Gtk
{

void ScrolledWindow::add(Gtk::Widget& widget)
{
  GtkWidget* gwidget = widget.gobj();

  //This check is courtesy of James Henstridge on gtk-devel-list@gnome.org.
  if( GTK_WIDGET_GET_CLASS(gwidget)->set_scroll_adjustments_signal == 0)
  {
    //It doesn't have native scrolling capability, so it should be put inside a viewport first:
    gtk_scrolled_window_add_with_viewport(gobj(), gwidget);
  }
  else
  {
    //It can work directly with a GtkScrolledWindow, so just use the Container::add():
    Bin::add(widget);
  }
}

} //namespace Gtk


namespace
{


void ScrolledWindow_signal_scroll_child_callback(GtkScrolledWindow*, GtkScrollType p0,gboolean p1,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot2<void,ScrollType,bool> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (((ScrollType)(p0))
,p1
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo ScrolledWindow_signal_scroll_child_info =
{
  "scroll_child",
  (GCallback) &ScrolledWindow_signal_scroll_child_callback,
  (GCallback) &ScrolledWindow_signal_scroll_child_callback
};


void ScrolledWindow_signal_move_focus_out_callback(GtkScrolledWindow*, GtkDirectionType p0,void* data)
{
  using namespace Gtk;
  typedef SigC::Slot1<void,DirectionType> SlotType;

  try
  {
    SigC::SlotNode *const slot = Glib::SignalProxyNormal::data_to_slot(data);
    (*(SlotType::Proxy)(slot->proxy_))
        (((DirectionType)(p0))
, slot);
  }
  catch(...)
  {
    Glib::exception_handlers_invoke();
  }
}

const Glib::SignalProxyInfo ScrolledWindow_signal_move_focus_out_info =
{
  "move_focus_out",
  (GCallback) &ScrolledWindow_signal_move_focus_out_callback,
  (GCallback) &ScrolledWindow_signal_move_focus_out_callback
};

} // anonymous namespace


namespace Glib
{

Gtk::ScrolledWindow* wrap(GtkScrolledWindow* object, bool take_copy /* = false */)
{
  return dynamic_cast<Gtk::ScrolledWindow *> (Glib::wrap_auto ((GObject*)(object), take_copy));
}

} /* namespace Glib */

namespace Gtk
{


/* The *_Class implementation: */

GType ScrolledWindow_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_scrolled_window_get_type(), "gtkmm__ScrolledWindow", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void ScrolledWindow_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
  klass->scroll_child = &scroll_child_callback;
  klass->move_focus_out = &move_focus_out_callback;
}


void ScrolledWindow_Class::scroll_child_callback(GtkScrolledWindow* self, GtkScrollType p0, gboolean p1)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_scroll_child(((ScrollType)(p0))
, p1
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->scroll_child)
      (*base->scroll_child)(self, p0, p1);
  }
}

void ScrolledWindow_Class::move_focus_out_callback(GtkScrolledWindow* self, GtkDirectionType p0)
{
  CppObjectType *const obj = dynamic_cast<CppObjectType*>(
      Glib::ObjectBase::_get_current_wrapper((GObject*)self));

  if(obj)
  {
    try
    {
      obj->on_move_focus_out(((DirectionType)(p0))
);
    }
    catch(...)
    {
      Glib::exception_handlers_invoke();
    }
  }
  else
  {
    BaseClassType *const base = static_cast<BaseClassType*>(
        g_type_class_peek_parent(G_OBJECT_GET_CLASS(self)) // Get the parent class of the object class.
);
    g_assert(base != 0);

    if(base->move_focus_out)
      (*base->move_focus_out)(self, p0);
  }
}


Glib::ObjectBase* ScrolledWindow_Class::wrap_new(GObject* o)
{
  return manage(new ScrolledWindow((GtkScrolledWindow*)(o)));
}


/* The implementation: */

ScrolledWindow::ScrolledWindow(GtkScrolledWindow* castitem)
: Gtk::Bin((GtkBin*)(castitem))
{}

ScrolledWindow::~ScrolledWindow()
{
  destroy_();
}

ScrolledWindow::CppClassType ScrolledWindow::scrolledwindow_class_; //Initialize static member.

GType ScrolledWindow::get_type()
{
  return scrolledwindow_class_.get_type();
}

GType ScrolledWindow::get_base_type()
{
  return gtk_scrolled_window_get_type();
}

ScrolledWindow::ScrolledWindow()
: Gtk::Bin((GtkBin*)(g_object_new(get_type(), (char*)0)))
{}

ScrolledWindow::ScrolledWindow(Adjustment& hadjustment, Adjustment& vadjustment)
: Gtk::Bin((GtkBin*)( g_object_new(get_type(), "hadjustment", (hadjustment).gobj(), "vadjustment", (vadjustment).gobj(), (char*)0) ))
{}

void ScrolledWindow::set_hadjustment(Gtk::Adjustment* hadjustment)
{
  gtk_scrolled_window_set_hadjustment(gobj(), (GtkAdjustment*)Glib::unwrap(hadjustment));
}

void ScrolledWindow::set_vadjustment(Gtk::Adjustment* vadjustment)
{
  gtk_scrolled_window_set_vadjustment(gobj(), (GtkAdjustment*)Glib::unwrap(vadjustment));
}

void ScrolledWindow::set_hadjustment(Gtk::Adjustment& hadjustment)
{
  gtk_scrolled_window_set_hadjustment(gobj(), (hadjustment).gobj());
}

void ScrolledWindow::set_vadjustment(Gtk::Adjustment& vadjustment)
{
  gtk_scrolled_window_set_vadjustment(gobj(), (vadjustment).gobj());
}

Gtk::Adjustment* ScrolledWindow::get_hadjustment() const
{
  return Glib::wrap(gtk_scrolled_window_get_hadjustment(const_cast<GtkScrolledWindow*>(gobj())));
}

Gtk::Adjustment* ScrolledWindow::get_vadjustment() const
{
  return Glib::wrap(gtk_scrolled_window_get_vadjustment(const_cast<GtkScrolledWindow*>(gobj())));
}

void ScrolledWindow::set_policy(PolicyType hscrollbar_policy, PolicyType vscrollbar_policy)
{
  gtk_scrolled_window_set_policy(gobj(), ((GtkPolicyType)(hscrollbar_policy)), ((GtkPolicyType)(vscrollbar_policy)));
}

void ScrolledWindow::get_policy(PolicyType& hscrollbar_policy, PolicyType& vscrollbar_policy) const
{
  gtk_scrolled_window_get_policy(const_cast<GtkScrolledWindow*>(gobj()), ((GtkPolicyType*) &(hscrollbar_policy)), ((GtkPolicyType*) &(vscrollbar_policy)));
}

void ScrolledWindow::set_placement(CornerType window_placement)
{
  gtk_scrolled_window_set_placement(gobj(), ((GtkCornerType)(window_placement)));
}

CornerType ScrolledWindow::get_placement() const
{
  return ((CornerType)(gtk_scrolled_window_get_placement(const_cast<GtkScrolledWindow*>(gobj()))));
}

void ScrolledWindow::set_shadow_type(ShadowType type)
{
  gtk_scrolled_window_set_shadow_type(gobj(), ((GtkShadowType)(type)));
}

ShadowType ScrolledWindow::get_shadow_type() const
{
  return ((ShadowType)(gtk_scrolled_window_get_shadow_type(const_cast<GtkScrolledWindow*>(gobj()))));
}

bool ScrolledWindow::get_vscrollbar_visible() const
{
  return gobj()->hscrollbar_visible;
}

bool ScrolledWindow::get_hscrollbar_visible() const
{
  return gobj()->vscrollbar_visible;
}

Gtk::HScrollbar* ScrolledWindow::get_hscrollbar()
{
  return Glib::wrap((GtkHScrollbar*)(gobj()->hscrollbar));
}

const Gtk::HScrollbar* ScrolledWindow::get_hscrollbar() const
{
  return Glib::wrap((GtkHScrollbar*)(gobj()->hscrollbar));
}

Gtk::VScrollbar* ScrolledWindow::get_vscrollbar()
{
  return Glib::wrap((GtkVScrollbar*)(gobj()->vscrollbar));
}

const Gtk::VScrollbar* ScrolledWindow::get_vscrollbar() const
{
  return Glib::wrap((GtkVScrollbar*)(gobj()->vscrollbar));
}


Glib::SignalProxy2<void,ScrollType,bool> ScrolledWindow::signal_scroll_child()
{
  return Glib::SignalProxy2<void,ScrollType,bool>(this, &ScrolledWindow_signal_scroll_child_info);
}

Glib::SignalProxy1<void,DirectionType> ScrolledWindow::signal_move_focus_out()
{
  return Glib::SignalProxy1<void,DirectionType>(this, &ScrolledWindow_signal_move_focus_out_info);
}


Glib::PropertyProxy<Gtk::Adjustment*> ScrolledWindow::property_hadjustment()
{
  return Glib::PropertyProxy<Gtk::Adjustment*>(this, "hadjustment");
}

Glib::PropertyProxy<Gtk::Adjustment*> ScrolledWindow::property_vadjustment()
{
  return Glib::PropertyProxy<Gtk::Adjustment*>(this, "vadjustment");
}

Glib::PropertyProxy<PolicyType> ScrolledWindow::property_hscrollbar_policy()
{
  return Glib::PropertyProxy<PolicyType>(this, "hscrollbar_policy");
}

Glib::PropertyProxy<PolicyType> ScrolledWindow::property_vscrollbar_policy()
{
  return Glib::PropertyProxy<PolicyType>(this, "vscrollbar_policy");
}

Glib::PropertyProxy<CornerType> ScrolledWindow::property_window_placement()
{
  return Glib::PropertyProxy<CornerType>(this, "window_placement");
}

Glib::PropertyProxy<ShadowType> ScrolledWindow::property_shadow_type()
{
  return Glib::PropertyProxy<ShadowType>(this, "shadow_type");
}


void Gtk::ScrolledWindow::on_scroll_child(ScrollType scroll, bool horizontal)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->scroll_child)
    (*base->scroll_child)(gobj(),((GtkScrollType)(scroll)),static_cast<int>(horizontal));
}

void Gtk::ScrolledWindow::on_move_focus_out(DirectionType direction)
{
  BaseClassType *const base = static_cast<BaseClassType*>(
      g_type_class_peek_parent(G_OBJECT_GET_CLASS(gobject_)) // Get the parent class of the object class.
);
  g_assert(base != 0);

  if(base->move_focus_out)
    (*base->move_focus_out)(gobj(),((GtkDirectionType)(direction)));
}


} // namespace Gtk

