
#include <gtkmm/settings.h>
#include <gtkmm/private/settings_p.h>

// -*- c++ -*-
/* $Id: settings.ccg,v 1.1 2001/10/21 03:52:26 murrayc Exp $ */

/* 
 *
 * Copyright 1998-1999 The Gtk-- Development Team
 * Copyright 2001      Free Software Foundation
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

namespace Gtk
{

Glib::RefPtr<Settings> Settings::get_default()
{
  return Glib::RefPtr<Settings>( new Settings( gtk_settings_get_default() ) );
}


} //namespace Gtk


namespace
{
} // anonymous namespace


namespace Glib
{

Glib::RefPtr<Gtk::Settings> wrap(GtkSettings* object, bool take_copy /* = false */)
{
  return Glib::RefPtr<Gtk::Settings>( dynamic_cast<Gtk::Settings*> (Glib::wrap_auto ((GObject*)(object), take_copy)) );
  //We use dynamic_cast<> in case of multiple inheritance.
}

} /* namespace Glib */


namespace Gtk
{


/* The *_Class implementation: */

GType Settings_Class::get_type()
  {
    if (!gtype_) //Create the GType if necessary.
      {
        //Make sure that the parent type has been created:
        CppClassParent::CppObjectType::get_type();

        //Create a derived C type.
        //These init funcs will, in turn, call the init functions in the base C type.
        GTypeInfo info =
          {
            sizeof(BaseClassType), /* class_size */
            0, /* base_init */
            0, /* base_finalize */
            (GClassInitFunc) &class_init_function, /* class_init */
            0, /* class_finalize */
            0, /* class_data */
            sizeof(BaseObjectType), /* instance_size */
            0, /* n_preallocs */
            0, /* instance_init */
	    0 /* value_table */
          };

        gtype_ = g_type_register_static(gtk_settings_get_type(), "gtkmm__Settings", &info, GTypeFlags(0));

  //Add derived versions of interfaces, if the C type implements any interfaces:
      }
    return gtype_;
  }
void Settings_Class::class_init_function(BaseClassType* klass)
{
  CppClassParent::class_init_function((CppClassParent::BaseClassType*) klass);
}


Glib::ObjectBase* Settings_Class::wrap_new(GObject* o)
{
  return new Settings((GtkSettings*)(o));
}


/* The implementation: */

GtkSettings* Settings::gobj_copy()
{
  reference();
  return gobj();
}

Glib::RefPtr<Settings> Settings::wrap_specific_type(GtkSettings* gobject, bool take_copy /* = false */) //static
{
  Glib::RefPtr<Settings> refPtr;

  if(gobject)
  {
    //Check for an existing wrapper:
    Settings* pCppObject = dynamic_cast<Settings*>(Glib::ObjectBase::_get_current_wrapper(G_OBJECT(gobject)));
    if(pCppObject)
    {
      //Return the existing wrapper:
      refPtr = Glib::RefPtr<Settings>(pCppObject);
    }
    else
    {
      //Create a new wrapper:
      refPtr = Glib::RefPtr<Settings>( new Settings(gobject) );
    }

    if(take_copy && refPtr)
      refPtr->reference();
  }

  return refPtr;
}

Settings::Settings(GtkSettings* castitem)
: Glib::Object((GObject*)(castitem))
{}

Settings::~Settings()
{
}

Settings::CppClassType Settings::settings_class_; //Initialize static member.

GType Settings::get_type()
{
  return settings_class_.get_type();
}

GType Settings::get_base_type()
{
  return gtk_settings_get_type();
}


Glib::PropertyProxy<int> Settings::property_gtk_double_click_time()
{
  return Glib::PropertyProxy<int>(this, "gtk_double_click_time");
}

Glib::PropertyProxy<bool> Settings::property_gtk_cursor_blink()
{
  return Glib::PropertyProxy<bool>(this, "gtk_cursor_blink");
}

Glib::PropertyProxy<int> Settings::property_gtk_cursor_blink_time()
{
  return Glib::PropertyProxy<int>(this, "gtk_cursor_blink_time");
}

Glib::PropertyProxy<bool> Settings::property_gtk_split_cursor()
{
  return Glib::PropertyProxy<bool>(this, "gtk_split_cursor");
}

Glib::PropertyProxy<Glib::ustring> Settings::property_gtk_theme_name()
{
  return Glib::PropertyProxy<Glib::ustring>(this, "gtk_theme_name");
}

Glib::PropertyProxy<Glib::ustring> Settings::property_gtk_key_theme_name()
{
  return Glib::PropertyProxy<Glib::ustring>(this, "gtk_key_theme_name");
}

Glib::PropertyProxy<Glib::ustring> Settings::property_gtk_menu_bar_accel()
{
  return Glib::PropertyProxy<Glib::ustring>(this, "gtk_menu_bar_accel");
}

Glib::PropertyProxy<ToolbarStyle> Settings::property_gtk_toolbar_style()
{
  return Glib::PropertyProxy<ToolbarStyle>(this, "gtk_toolbar_style");
}

Glib::PropertyProxy<IconSize> Settings::property_gtk_toolbar_icon_size()
{
  return Glib::PropertyProxy<IconSize>(this, "gtk_toolbar_icon_size");
}

Glib::PropertyProxy<bool> Settings::property_gtk_can_change_accels()
{
  return Glib::PropertyProxy<bool>(this, "gtk_can_change_accels");
}

Glib::PropertyProxy<bool> Settings::property_gtk_entry_select_on_focus()
{
  return Glib::PropertyProxy<bool>(this, "gtk_entry_select_on_focus");
}

Glib::PropertyProxy<Glib::ustring> Settings::property_gtk_color_palette()
{
  return Glib::PropertyProxy<Glib::ustring>(this, "gtk_color_palette");
}


} // namespace Gtk

